# TODO #2 Create Exception Class and make test
class ExceedException(Exception):
  pass


class UserNotFoundException(Exception):
  pass


class ProblemNotFoundException(Exception):
  pass


class UnknownSiteException(Exception):
  pass
